

#include <eZ8.h>

////////////////////////////////////////////////////////////
// Servo Driver/Controller Software Interface
// (C) 2007 by e-Gizmo Mechatronix Central

// low level function prototypes
void clockin(void);
void servo_out(int mot,int com,int param);


//// ***************************************************************
//   High Level Commands
//

//  ----------------------------------------------------------------
//  move to position 
//  mot - motor number that where the move command will take effect
//  pos - position in degrees. Valid position range is -90 to +90 degrees
//  function returns with 0 if command is succesful

char	move(int mot,int pos,int speed)
{
	if((pos >90) | (pos<-90)) return 1;		// exit with 0 if pos is out of range
	speed=speed+128;		//speed 129 minimum

	pos=(pos*100/90)+150;	// convert to degrees

	servo_out(mot,2,pos);
	servo_out(mot,1,speed);
	return 0;
}

// --------------------------------------------------------------------
// Adjust speed
//  mot - motor number that where the move command will take effect
//  speed - speed argument, 0 (fastest) to 127(slowest).
// 	you can call this command at anytime even while the motor is moving
//  for an instantaneous change in speed
// 	See the Servo Driver/Controller Technical manual for a complete description

char speed(int mot,int speed)
{
	if(speed>127) return 0;
	speed=speed+128;		// 128 is the internal zero reference of the servo driver
	servo_out(mot,1,speed);
	return 1;				// exit with 1 to indicate successful speed change	
}


// --------------------------------------------------------------------
// Servo ON/OFF
//  mgroup - motor group (8motors) where the ON/OFF command will take effect
//  mgroup = 1 affects motor 1 to 8
//  mgroup = 9 affects motor 9 to 16
// 	See the Servo Driver/Controller Technical manual for a complete description

void servo_ON(int mgroup)
{
		servo_out(mgroup,8,0xFF);			// Turn ON servo output

}

void servo_OFF(int mgroup)
{
		servo_out(mgroup,8,0x00);			// Turn ON servo output

}

//**********************************************************
// Low level Function: Generate 1 clock pulse
// Clock in data to async port
void clockin(void)
{
	PBOUT &=0xDF;		//clock low
	while((PBIN &0x10)==0x10);	// wait until ack received
	PBOUT |= 0x20;
	while((PBIN &0x10)==0);		//wait until ack is high
}


////////////////////////////////////////////////////////////////
// Low level Function : Send Command to servo driver/controller
//		mot - motor number where the command will take effect
//		param - command parameter/argument
//		com - command 
//			1- set speed 2- set position 3 - all neutral 8 - servo ON/OFF
// See the Servo Driver/Controller Technical manual for a complete description
// of these low level commands


void servo_out(int mot,int com,int param)
{

	char	motcom;
	int	i;

	// select servo driver IC
	// If motor < 9, connect to servo driver on JP2 and select servo driver U2
	if(mot<9) PBOUT &= 0xFE;

	// If motor = 9 to 16 connect to servo driver on JP2 and select servo driver U3
	if(mot>=9)
		if(mot<17) 
			{
			PBOUT &=0xFD;
			mot=mot-8;
			}

	// If motor = 17 to 24 connect to servo driver on JP3 and select servo driver U2
	if(mot>=17)
		if(mot<25) 
			{
			PBOUT &=0xFB;
			mot=mot-16;
			}
	// If motor >= 25, connect to servo driver on JP3 and select servo driver U3
	if(mot>= 25) 
		{
		PBOUT &=0xF7;
		mot=mot-24;
		}

	//merge command and motor number into single byte
	// with motor number occupying high 4 bits
	// and command occupying lower 4 bits

	motcom=mot;
	motcom <<=4;		// transfer to high nibble
	motcom &= 0xF0;
	motcom = motcom | com; // mix command field

	// send parameter 
	for(i=1;i<9;++i)
	{
		if((param & 0x01)==1)		// output a 1 if lsb is 1
			PDOUT |= 0x01;
			else
			PDOUT &= 0xFE;			// else output a zero
		param >>=1;					// shift right 1 bit position
		clockin();					// clock to transfer data

	}

	// send motor number and command
	// algorithm same as send parameter
	for(i=1;i<9;++i)
	{	
		if((motcom &0x01)==0x01) 
			PDOUT |= 0x01;
			else
			PDOUT &= 0xFE;

		motcom >>=1;
		clockin();
	}

		PBOUT |= 0x0F;					// deselect servo drivers
}



